#
# vocalips041.tcl - GUI
#
# A GUI wrapper written in Tcl/Tk. 
# It wraps the command-line processor (actually a Perl script
# "vsqtxt2lipsync.pl") that does the processing. 
# 
# 1) The user specifies the VSQ file name.
# 2) The wrapper calls mf2t, and then vsqtxt2lipsync,
#    to display the track names and the BPM map. 
# 3) The user specifies the output mode, and the tracks to be
#    processed. Note that more than one tracks can be processed.
# 4) The wrapper calls vsqtxt2lipsync for the second time,
#    to obtain the results.
# 
# That's totally fine to run vsqtxt2lipsync manually in a
# console. All the options are described at the beginning 
# of the script. 
#
# preview window is almost working, but japanese chars are
# not showing. very weird. this is put off.
#
# Rev.0.41  Apr. 01, 2008  Aprilfool version: Miku/Rin/Len color
#                          pallette:-)
# Rev.0.39  Mar. 09, 2008  add beat track. 
# Rev.0.30  Dec. 24, 2007  add BPM-map preview.
# Rev.0.21  Nov. 27, 2007  just for the update to vsqtxt2lipsync_042.
# Rev.0.20  Nov. 23, 2007
# Rev.0.10  Nov. 3, 2007
# 
# author: zhuo   www.geocities.jp/zhuoware
# 

set VocalipsRevision "0.41"
set Vsqtxt2lipsyncScript "bin/vsqtxt2lipsync_050.pl"

set w .top
catch {destroy $w}
toplevel $w
wm title $w "Vocalips Rev.$VocalipsRevision"
wm iconname $w "Vocalips Rev.$VocalipsRevision"
wm withdraw .

set vocalipsAttr(fixedFont) {FixedSys 8}


proc createFileEntry { path text cmd } {
    frame $path
    label $path.lab -width 16 -text $text -anchor e
    entry $path.ent -width 80
    button $path.but -text "Browse ..." -command "$cmd $path"
    pack $path.lab -side left
    pack $path.ent -side left -expand yes -fill x
    pack $path.but -side left
    pack $path -fill x -padx 1c -pady 3
}

proc setEntryString { path string } {
    if {[string compare $string ""]} {
	$path delete 0 end
	$path insert 0 $string
	$path xview end
    }
}

proc consPuts { str } {
    global w
    catch {$w.cons.text insert end "$str"}
}

if { 0 } {
    frame $w.cons
    scrollbar $w.cons.scroll -command "$w.cons.text yview"
    text $w.cons.text -height 10 -yscroll "$w.cons.scroll set"
    pack $w.cons.scroll -side right -expand no -fill y
    pack $w.cons.text -side left -expand yes -fill both
    pack $w.cons -side bottom  -expand yes -fill both
}




createFileEntry $w.vsqfile "vocaloid file: " "vsqFileChooser $w.outfile.ent"
createFileEntry $w.outfile "output file: " "outFileChooser"
createFileEntry $w.tblfile "phoneme conv. file: " "tblFileChooser"

$w.tblfile.ent delete 0 end
$w.tblfile.ent insert 0 [lindex [glob -nocomplain -types f "*.tbl"] 0] 
$w.tblfile.ent xview end


labelframe $w.bpmmap -text "bpm map" -padx 5 -pady 5
scrollbar $w.bpmmap.scroll -command "$w.bpmmap.list yview"

text $w.bpmmap.list -yscroll "$w.bpmmap.scroll set" -setgrid 1 -width 24 -height 8 -relief flat -font FixedSys  -state disabled

#    listbox $w.bpmmap.list -yscroll "$w.bpmmap.scroll set" -setgrid 1 -height 8 -selectmode single -font $vocalipsAttr(fixedFont)



pack $w.bpmmap.scroll -side right -fill y
pack $w.bpmmap.list -side left -expand yes -fill both 
pack $w.bpmmap -side left -expand yes -fill both  -padx 3m


frame $w.settings
labelframe $w.settings.tracks -text "track chooser" -padx 5 -pady 5
scrollbar $w.settings.tracks.scroll -command "$w.settings.tracks.list yview"
listbox $w.settings.tracks.list -yscroll "$w.settings.tracks.scroll set" -setgrid 1 -height 8 -selectmode extended
pack $w.settings.tracks.scroll -side right -fill y
pack $w.settings.tracks.list -side left -expand yes -fill both 
pack $w.settings.tracks -side left -expand yes -fill both  -padx 3m


labelframe $w.settings.mode -text "output mode" -padx 5 -pady 5
radiobutton $w.settings.mode.e2 -text "standard" -variable outputMode -relief flat -value "e2" -anchor w
radiobutton $w.settings.mode.e1 -text "event-base, \"sil\" when early" -variable outputMode -relief flat -value "e1" -anchor w
radiobutton $w.settings.mode.e0 -text "event-base, always with \"end\"" -variable outputMode -relief flat -value "e0" -anchor w
radiobutton $w.settings.mode.f -text "frame-base" -variable outputMode -relief flat -value "f" -anchor w
radiobutton $w.settings.mode.v -text "raw vsq text" -variable outputMode -relief flat -value "v" -anchor w
pack $w.settings.mode.e2 $w.settings.mode.e1 $w.settings.mode.e0 $w.settings.mode.f $w.settings.mode.v -side top -pady 2 -expand no -fill x
pack $w.settings.mode -side left -expand no -fill y  -padx 3m
$w.settings.mode.e2 invoke

labelframe $w.settings.fb -text "options for frame-base mode" -padx 5 -pady 5
label $w.settings.fb.fpslab -text "fps"
entry $w.settings.fb.fpsent -width 8
$w.settings.fb.fpsent insert 0 "30.00"
label $w.settings.fb.offsetlab -text "offset(msec)"
entry $w.settings.fb.offsetent -width 8
$w.settings.fb.offsetent insert 0 "0.0"
pack $w.settings.fb.fpslab $w.settings.fb.fpsent $w.settings.fb.offsetlab $w.settings.fb.offsetent -side left -expand yes -padx 2m
pack $w.settings.fb -side top -fill none -padx 3m

labelframe $w.settings.msg -text "notes" -padx 5 -pady 5
label $w.settings.msg.note -text "Vocalips Rev.$VocalipsRevision\t\tApril 1, 2008

- Click this area to enjoy color variations!
" -anchor nw -justify left

pack $w.settings.msg.note -side top -fill both -expand yes -padx 2m
pack $w.settings.msg -side top -fill both -expand yes  -padx 3m
pack $w.settings -side top -expand yes -fill both

frame $w.buttons
#checkbutton $w.buttons.doPreview -text "preview"
button $w.buttons.convert -width 16 -text "convert" -command "convert $w"
button $w.buttons.quit -width 16 -text quit -command "destroy $w; exit"
#pack $w.buttons.doPreview -side left -expand 1
pack $w.buttons.convert -side left -expand 1
pack $w.buttons.quit -side left -expand 1
pack $w.buttons -side bottom -fill x -pady 5m


set colorModeIndex -1
set colorModes { None Miku RinLen }
array set colorValues {
    Miku,MonoDark \#373b3e
    Miku,MonoMid \#bec8d1
    Miku,MonoLight \#fffeec
    Miku,Col1Light \#c3e5e7
    Miku,Col1Mid \#86cecb
    Miku,Col1Dark \#137a7f
    Miku,Col2 \#e12885
    RinLen,MonoDark \#555658
    RinLen,MonoMid \#b7b7b7
    RinLen,MonoLight \#ffffff
    RinLen,Col1Light \#fffbe2
    RinLen,Col1Mid \#fcf5a7
    RinLen,Col1Dark \#ffd98c
    RinLen,Col2 \#2b90a2
    None,MonoDark \#ecead9
    None,MonoMid \#ecead9
    None,MonoLight \#ffffff
    None,Col1Light \#ecead9
    None,Col1Mid \#ecead9
    None,Col1Dark \#ecead9
    None,Col2 \#ecead9
}
#color info from: http://smallwebmemo.blog113.fc2.com/blog-entry-156.html

bind $w.settings.msg.note <1> {
    setColorMode $w
}

proc setColorMode {w} {
    global colorModeIndex colorModes colorValues
    incr colorModeIndex
    if {$colorModeIndex >= [llength $colorModes]} {
	set colorModeIndex 0
    }
    set mode [lindex $colorModes $colorModeIndex]

    setColorsRecursive [winfo toplevel $w] $mode
}

proc setColorsRecursive {w mode} {
    global colorValues
#    consPuts [winfo class $w]
    switch -exact [winfo class $w] {
	Toplevel {
	    $w configure -background $colorValues($mode,Col1Dark)
	}
	Frame -
	Labelframe {
	    $w configure -background $colorValues($mode,Col1Mid)
	}
	Label {
	    $w configure -background $colorValues($mode,Col1Light)
	}
	Button -
	Radiobutton {
	    $w configure -background $colorValues($mode,Col1Light) \
		-activebackground $colorValues($mode,Col2)
	}
	Entry -
	Listbox -
	Text {
	    $w configure -background $colorValues($mode,MonoLight)
	}
	Scrollbar {
	    $w configure -background $colorValues($mode,MonoMid)
	}
    }
    foreach s [winfo children $w] {
	setColorsRecursive $s $mode
    }
}

setColorMode $w


proc getSelectedTracks {} {
    global w
    return [$w.settings.tracks.list curselection]
}


## the return value is a list like:
##     { A B }
## where  A is the seleted tracks, either: 
##     {}          # no tracks specified
## or 
##     { 0 1 3 }   # these are specified
## B is 0  when the beat track is NOT specified, 1 otherwise.
##
proc getSelectedTracksInDetail {} {
    global w
    set maxIdx [expr [$w.settings.tracks.list size]-1]
    set tracks [$w.settings.tracks.list curselection]
    set realTracks [lsearch -all -inline -exact -not $tracks $maxIdx]
    set hasBeatTrack [llength [lsearch -inline -exact $tracks $maxIdx]]
    return [list $realTracks $hasBeatTrack]
}


proc setTrackList {trackListFile} {
    global w Vsqtxt2lipsyncScript
    set listpath $w.settings.tracks.list

    set fd [open "$trackListFile"]
    $listpath delete 0 end
    while {[gets $fd line] >= 0 } {
	set elms [split $line "\t"]
	$listpath insert end [lindex $elms 1]
    }
    close $fd
## new in 0.39
    $listpath insert end "*BEAT*"
## note:  japanese characters were not shown....
##        this might take place when track names are in Japanese, too.
##        we would need care.
## end
    $listpath selection set 0 end
}



proc setBPMMap {bpmFile} {
    global w Vsqtxt2lipsyncScript
    set listpath $w.bpmmap.list

    set fd [open "$bpmFile"]
    set isSection 0

    $listpath configure -state normal
    $listpath delete 0.0 end
#	$listpath delete 0 end

    $listpath insert end " time(sec)       BPM\n"
    $listpath insert end "--------------------\n"
    while {[gets $fd line] >= 0 } {
	set elms [split $line "\t"]
	if { [lindex $elms 0] == "\#\#" && [lindex $elms 2] == "bpm_map" } {
	    if { [lindex $elms 1] == "begin" } {
		set isSection 1
		continue
	    } elseif { [lindex $elms 1] == "end" } {
		set isSection 0
		continue
	    }
	}
	if { $isSection == 1 } {
	    $listpath insert end [join $elms ""]
	    $listpath insert end "\n"
#	    $listpath insert end $line	
	}
    }
    close $fd
    $listpath configure -state disabled
}



proc vsqFileChooser { out_ent path } {
    global w Vsqtxt2lipsyncScript
    set types {{"vocaloid files" {.vsq}} {"All files" *}}
    set file "[tk_getOpenFile -filetypes $types -parent $path -initialfile [$path.ent get]]"
    if { "$file" == "" || ! [file exists "$file"] } {
	return
    }
    regsub {\.vsq$} "$file" ".txt" outfile 
    setEntryString $path.ent "$file"
    setEntryString $out_ent "$outfile"

    file copy -force "$file" "T.VSQ"
    if { [catch {exec bin/mf2t "T.VSQ" "T.MFT"} msg ] } {
	tk_messageBox -icon info -message "mf2t error: $msg" -type ok -parent $w
	catch {file delete "T.VSQ" "T.MFT"}
	return
    }
    if { [catch {exec bin/Perl $Vsqtxt2lipsyncScript -m t < "T.MFT" > "T1.TXT"} msg ] } {
	tk_messageBox -icon info -message "track-info error: $msg" -type ok -parent $w
	catch {file delete "T.VSQ" "T.MFT" "T1.TXT"}
	return
    }
    setTrackList "T1.TXT"
    if { [catch {exec bin/Perl $Vsqtxt2lipsyncScript -m m < "T.MFT" > "T2.TXT"} msg] } {
	tk_messageBox -icon info -message "bpm-info error: $msg" -type ok -parent $w
	catch {file delete "T.VSQ" "T.MFT" "T1.TXT" "T2.TXT"}
	return
    }
    setBPMMap "T2.TXT"
    file delete "T.VSQ" "T.MFT" "T1.TXT" "T2.TXT"
}

proc tblFileChooser { path } {
    set types {{"table files" {.tbl}} {"All files" *}}
    setEntryString $path.ent "[tk_getOpenFile -filetypes $types -parent $path -initialfile [$path.ent get]]"
}

proc outFileChooser { path } {
    set types {{"text files" {.txt}} {"All files" *}}
    setEntryString $path.ent "[tk_getSaveFile -filetypes $types -parent $path -initialfile [$path.ent get]]"
}


proc convert { w } {
    global outputMode Vsqtxt2lipsyncScript
# global doPreview    
    set vsqfile "[$w.vsqfile.ent get]"
    set tblfile "[$w.tblfile.ent get]"
    set outfile "[$w.outfile.ent get]"

    set fps [$w.settings.fb.fpsent get]
    set offset [$w.settings.fb.offsetent get]

    set trackInfo [getSelectedTracksInDetail]
    set tracks [join [lindex $trackInfo 0] ","]
    if { $tracks == "" } {
	set tracks "none"
    }
    set beatOption ""
    if { [lindex $trackInfo 1] == 1 } {
	set beatOption "-b"
    }
    if { "$vsqfile" == "" || ! [file exists "$vsqfile"] } {
	tk_messageBox -icon info -message "Please choose a vocaloid file." -type ok -parent $w
	return
    }
    if { "$tblfile" == "" || ! [file exists "$tblfile"] } {
	tk_messageBox -icon info -message "Please choose a table file." -type ok -parent $w
	return
    }
    if { "$outfile" == "" } {
	tk_messageBox -icon info -message "Please choose an output file." -type ok -parent $w
	return
    }
    if { $outputMode == "" } {
	tk_messageBox -icon info -message "Please choose an output mode." -type ok -parent $w
	return
    }

    if { (![file exists "$outfile"]) || ([tk_messageBox -icon question -message "file exists: overwrite? $outfile" -type okcancel -parent $w] == "ok") } {
	file copy -force "$vsqfile" "T.VSQ"

## new 0.39:
	set cmd "exec bin/mf2t T.VSQ | "
	append cmd "bin/Perl \"$Vsqtxt2lipsyncScript\" -t \"$tracks\" "
	if {$beatOption != ""} { append cmd "$beatOption " }
	append cmd "-m \"$outputMode\" -p \"$tblfile\" \"$fps\" \"$offset\" "
	append cmd "> \"T.TXT\""
#	consPuts $cmd
	if { [catch {eval $cmd} msg] } {
	    tk_messageBox -icon info -message "conversion error:\n$msg\n" -type ok -parent $w
	    return
	}
## before 0.39:
#	if { [catch {exec bin/mf2t "T.VSQ" | bin/Perl $Vsqtxt2lipsyncScript -t "$tracks" "$beatOption" -m "$outputMode" -p "$tblfile" "$fps" "$offset" > "T.TXT"} msg] } {
#	    tk_messageBox -icon info -message "conversion error:\n$msg\n" -type ok -parent $w
#	    return
#	}
## end

# wanted to do preview, but somehow 
# japanese chars were not shown correctly,
# although independent codes worked without problems.
# "english mode" being assumed may be affecting 
# the behavior.
	if { 0 } {
	    if {! [winfo exists $w.preview]} {
		toplevel $w.preview
                
		scrollbar $w.preview.scroll -command "$w.preview.text yview"
		pack $w.preview.scroll -side right -expand no -fill y
		text $w.preview.text -width 136 -height 48 -yscroll "$w.preview.scroll set"
		$w.preview.text configure -state normal
		pack $w.preview.text -side left -expand yes -fill both
		pack $w.preview -side bottom -expand yes -fill both
	    }
	    $w.preview.text delete 0.0 end
	    set previewfd [open "T.TXT"]
#OK	    consPuts "preview display\n"
#	    consPuts "vr[\\n"
	    consPuts [encoding convertfrom shiftjis "vr[\\n"]
	    while {[gets $previewfd line] >= 0} {
		append previewContent "$line" "\n"
	    }
	    close $previewfd
	    consPuts "$previewContent"
#		$w.preview.text insert end "$line\n"
#	    $w.preview.text configure -state disabled
	    update
	}
# end of "preview" codes.
	file copy -force "T.TXT" "$outfile"
	file delete "T.VSQ" "T.TXT"
	tk_messageBox -icon info -message "$outfile created.\n" -type ok -parent $w
    } else {
	tk_messageBox -icon info -message "save canceled.\n" -type ok -parent $w
    }
}

